import sys
from array import array
from collections import deque

from rollbar import DEFAULT_LOCALS_SIZES
from rollbar.lib import transforms
from rollbar.lib.transforms.shortener import ShortenerTransform
from rollbar.lib.type_info import Sequence
from rollbar.test import BaseTest


class TestClassWithAVeryVeryVeryVeryVeryVeryVeryLongName:
    pass


class ShortenerTransformTest(BaseTest):
    def setUp(self):
        self.data = {
            'string': 'x' * 120,
            'long': 17955682733916468498414734863645002504519623752387,
            'dict': {
                'one': 'one',
                'two': 'two',
                'three': 'three',
                'four': 'four',
                'five': 'five',
                'six': 'six',
                'seven': 'seven',
                'eight': 'eight',
                'nine': 'nine',
                'ten': 'ten',
                'eleven': 'eleven'
            },
            'list': [1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11],
            'tuple': (1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11),
            'set': set([1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11]),
            'frozenset': frozenset([1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11]),
            'array': array('l', [1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11]),
            'deque': deque([1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11], 15),
            'other': TestClassWithAVeryVeryVeryVeryVeryVeryVeryLongName()
        }

    def _assert_shortened(self, key, expected):
        shortener = ShortenerTransform(keys=[(key,)], **DEFAULT_LOCALS_SIZES)
        result = transforms.transform(self.data, shortener)

        if key == 'dict':
            self.assertEqual(expected, len(result))
        else:
            # the repr output can vary between Python versions
            stripped_result_key = result[key].strip("'\"u")

        if key == 'other':
            self.assertIn(expected, stripped_result_key)
        elif key != 'dict':
            self.assertEqual(expected, stripped_result_key)

        # make sure nothing else was shortened
        result.pop(key)
        self.assertNotIn('...', str(result))
        self.assertNotIn('...', str(self.data))

    def test_no_shorten(self):
        shortener = ShortenerTransform(**DEFAULT_LOCALS_SIZES)
        result = transforms.transform(self.data, shortener)
        self.assertEqual(self.data, result)

    def test_shorten_string(self):
        expected = '{}...{}'.format('x'*47, 'x'*48)
        self._assert_shortened('string', expected)

    def test_shorten_long(self):
        expected = '179556827339164684...5002504519623752387'
        self._assert_shortened('long', expected)

    def test_shorten_mapping(self):
        # here, expected is the number of key value pairs
        expected = 10
        self._assert_shortened('dict', expected)

    def test_shorten_list(self):
        expected = '[1, 2, 3, 4, 5, 6, 7, 8, 9, 10, ...]'
        self._assert_shortened('list', expected)

    def test_shorten_tuple(self):
        expected = '(1, 2, 3, 4, 5, 6, 7, 8, 9, 10, ...)'
        self._assert_shortened('tuple', expected)

    def test_shorten_set(self):
        expected = 'set([1, 2, 3, 4, 5, 6, 7, 8, 9, 10, ...])'
        if sys.version_info >= (3, 5):
            expected = '{1, 2, 3, 4, 5, 6, 7, 8, 9, 10, ...}'
        self._assert_shortened('set', expected)

    def test_shorten_frozenset(self):
        expected = 'frozenset([1, 2, 3, 4, 5, 6, 7, 8, 9, 10, ...])'
        if sys.version_info >= (3, 5):
            expected = 'frozenset({1, 2, 3, 4, 5, 6, 7, 8, 9, 10, ...})'
        self._assert_shortened('frozenset', expected)

    def test_shorten_array(self):
        expected = 'array(\'l\', [1, 2, 3, 4, 5, 6, 7, 8, 9, 10, ...])'
        if sys.version_info >= (3, 10):
            expected = '[1, 2, 3, 4, 5, 6, 7, 8, 9, 10, ...]'
        self._assert_shortened('array', expected)

    def test_shorten_deque(self):
        expected = 'deque([1, 2, 3, 4, 5, 6, 7, 8, 9, 10, ...])'
        if issubclass(deque, Sequence):
            expected = '[1, 2, 3, 4, 5, 6, 7, 8, 9, 10, ...]'
        self._assert_shortened('deque', expected)

    def test_shorten_other(self):
        expected = '<rollbar.test.test_shortener_transform.TestClas...'
        self._assert_shortened('other', expected)

    def test_shorten_object(self):
        data = {'request': {'POST': {i: i for i in range(12)}}}
        keys = [
                ('request', 'POST'),
                ('request', 'json'),
                ('body', 'request', 'POST'),
                ('body', 'request', 'json'),
                ]
        self.assertEqual(len(data['request']['POST']), 12)
        shortener = ShortenerTransform(keys=keys, **DEFAULT_LOCALS_SIZES)
        result = transforms.transform(data, shortener)
        self.assertEqual(type(result), dict)
        self.assertEqual(len(result['request']['POST']), 10)




import unittest


SNOWMAN = b'\xe2\x98\x83'
SNOWMAN_UNICODE = SNOWMAN.decode('utf8')


class BaseTest(unittest.TestCase):
    pass


def discover():
    loader = unittest.TestLoader()
    suite = loader.discover(__name__)
    return suite



import copy
import importlib
import json
import sys

from unittest import mock

try:
    import fastapi

    FASTAPI_INSTALLED = True
    ALLOWED_FASTAPI_VERSION = fastapi.__version__ >= '0.41.0'
except ImportError:
    FASTAPI_INSTALLED = False
    ALLOWED_FASTAPI_VERSION = False

import unittest

import rollbar
from rollbar.lib._async import AsyncMock
from rollbar.test import BaseTest


ALLOWED_PYTHON_VERSION = sys.version_info >= (3, 6)


@unittest.skipUnless(
    FASTAPI_INSTALLED and ALLOWED_PYTHON_VERSION, 'FastAPI requires Python3.6+'
)
class LoggingRouteUnsupportedFastAPIVersionTest(BaseTest):
    def test_should_disable_loading_route_handler_if_fastapi_is_too_old(self):
        import logging
        import fastapi
        from fastapi import FastAPI
        from fastapi.routing import APIRoute
        from rollbar.contrib.fastapi.routing import add_to as rollbar_add_to
        from rollbar.contrib.fastapi.utils import FastAPIVersionError

        logging.disable(logging.ERROR)  # silent logger for tests
        fastapi_version = fastapi.__version__

        app = FastAPI()
        old_route_class = app.router.route_class
        self.assertEqual(old_route_class, APIRoute)

        fastapi.__version__ = '0'
        with self.assertRaises(FastAPIVersionError):
            rollbar_add_to(app)

        fastapi.__version__ = '0.30.3'
        with self.assertRaises(FastAPIVersionError):
            rollbar_add_to(app)

        fastapi.__version__ = '0.40.10'
        with self.assertRaises(FastAPIVersionError):
            rollbar_add_to(app)

        self.assertEqual(app.router.route_class, old_route_class)

        logging.disable(logging.NOTSET)  # make sure logger is re-enabled
        fastapi.__version__ = fastapi_version


@unittest.skipUnless(
    FASTAPI_INSTALLED and ALLOWED_PYTHON_VERSION, 'FastAPI requires Python3.6+'
)
@unittest.skipUnless(ALLOWED_FASTAPI_VERSION, 'FastAPI v0.41.0+ is required')
class LoggingRouteTest(BaseTest):
    default_settings = copy.deepcopy(rollbar.SETTINGS)

    def setUp(self):
        importlib.reload(rollbar)
        rollbar.SETTINGS = copy.deepcopy(self.default_settings)
        rollbar.SETTINGS['handler'] = 'async'

    @mock.patch('rollbar.report_exc_info')
    def test_should_catch_and_report_errors(self, mock_report):
        from fastapi import FastAPI
        from rollbar.contrib.fastapi.routing import add_to as rollbar_add_to

        try:
            from fastapi.testclient import TestClient
        except ImportError:  # Added in FastAPI v0.51.0+
            from starlette.testclient import TestClient

        app = FastAPI()
        rollbar_add_to(app)

        @app.get('/')
        async def read_root():
            1 / 0

        client = TestClient(app)
        with self.assertRaises(ZeroDivisionError):
            client.get('/')

        mock_report.assert_called_once()

        args, kwargs = mock_report.call_args
        self.assertEqual(kwargs, {})

        exc_type, exc_value, exc_tb = args[0]

        self.assertEqual(exc_type, ZeroDivisionError)
        self.assertIsInstance(exc_value, ZeroDivisionError)

    @mock.patch('rollbar.report_exc_info')
    def test_should_report_with_request_data(self, mock_report):
        from fastapi import FastAPI
        from rollbar.contrib.fastapi.routing import add_to as rollbar_add_to

        try:
            from fastapi import Request
            from fastapi.testclient import TestClient
        except ImportError:  # Added in FastAPI v0.51.0+
            from starlette.requests import Request
            from starlette.testclient import TestClient

        app = FastAPI()
        rollbar_add_to(app)

        @app.get('/')
        def read_root():
            1 / 0

        client = TestClient(app)
        with self.assertRaises(ZeroDivisionError):
            client.get('/')

        mock_report.assert_called_once()
        request = mock_report.call_args[0][1]

        self.assertIsInstance(request, Request)

    @mock.patch('rollbar._check_config', return_value=True)
    @mock.patch('rollbar._serialize_frame_data')
    @mock.patch('rollbar.send_payload')
    def test_should_send_payload_with_request_data(self, mock_send_payload, *mocks):
        from fastapi import FastAPI
        from rollbar.contrib.fastapi.routing import add_to as rollbar_add_to

        try:
            from fastapi.testclient import TestClient
        except ImportError:  # Added in FastAPI v0.51.0+
            from starlette.testclient import TestClient

        app = FastAPI()
        rollbar_add_to(app)

        @app.get('/{path}')
        def read_root(path):
            1 / 0

        client = TestClient(app)
        with self.assertRaises(ZeroDivisionError):
            client.get('/test?param1=value1&param2=value2')

        mock_send_payload.assert_called_once()
        payload = mock_send_payload.call_args[0][0]
        payload_request = payload['data']['request']

        self.assertEqual(payload_request['method'], 'GET')
        self.assertEqual(payload_request['user_ip'], 'testclient')
        self.assertEqual(
            payload_request['url'],
            'http://testserver/test?param1=value1&param2=value2',
        )
        self.assertDictEqual(payload_request['params'], {'path': 'test'})
        self.assertDictEqual(
            payload_request['GET'], {'param1': 'value1', 'param2': 'value2'}
        )
        self.assertDictEqual(
            payload_request['headers'],
            {
                'accept': '*/*',
                'accept-encoding': 'gzip, deflate',
                'connection': 'keep-alive',
                'host': 'testserver',
                'user-agent': 'testclient',
            },
        )

    @mock.patch('rollbar._check_config', return_value=True)
    @mock.patch('rollbar._serialize_frame_data')
    @mock.patch('rollbar.send_payload')
    def test_should_send_payload_with_request_body(self, mock_send_payload, *mocks):
        from fastapi import Body, FastAPI
        from pydantic import BaseModel
        from rollbar.contrib.fastapi.routing import add_to as rollbar_add_to

        try:
            from fastapi.testclient import TestClient
        except ImportError:  # Added in FastAPI v0.51.0+
            from starlette.testclient import TestClient

        rollbar.SETTINGS['include_request_body'] = True
        expected_body = {'param1': 'value1', 'param2': 'value2'}

        app = FastAPI()
        rollbar_add_to(app)

        class TestBody(BaseModel):
            param1: str
            param2: str

        @app.post('/')
        def read_root(body: TestBody = Body(...)):
            1 / 0

        client = TestClient(app)
        with self.assertRaises(ZeroDivisionError):
            client.post('/', json=expected_body)

        mock_send_payload.assert_called_once()
        payload = mock_send_payload.call_args[0][0]
        payload_request = payload['data']['request']

        self.assertEqual(payload_request['method'], 'POST')
        self.assertEqual(payload_request['user_ip'], 'testclient')
        self.assertEqual(payload_request['url'], 'http://testserver/')
        self.assertEqual(payload_request['body'], json.dumps(expected_body))
        self.assertDictEqual(
            payload_request['headers'],
            {
                'accept': '*/*',
                'accept-encoding': 'gzip, deflate',
                'connection': 'keep-alive',
                'content-length': str(len(json.dumps(expected_body))),
                'content-type': 'application/json',
                'host': 'testserver',
                'user-agent': 'testclient',
            },
        )

    @mock.patch('rollbar._check_config', return_value=True)
    @mock.patch('rollbar._serialize_frame_data')
    @mock.patch('rollbar.send_payload')
    def test_should_send_payload_with_form_data(self, mock_send_payload, *mocks):
        from fastapi import FastAPI, Form
        from rollbar.contrib.fastapi.routing import add_to as rollbar_add_to

        try:
            from fastapi.testclient import TestClient
        except ImportError:  # Added in FastAPI v0.51.0+
            from starlette.testclient import TestClient

        expected_form = {'param1': 'value1', 'param2': 'value2'}
        expected_body = b'param1=value1&param2=value2'

        app = FastAPI()
        rollbar_add_to(app)

        @app.post('/')
        def read_root(param1: str = Form(...), param2: str = Form(...)):
            1 / 0

        client = TestClient(app)
        with self.assertRaises(ZeroDivisionError):
            r = client.post(
                '/',
                data=expected_body,
                headers={'Content-Type': 'application/x-www-form-urlencoded'},
            )

        mock_send_payload.assert_called_once()
        payload = mock_send_payload.call_args[0][0]
        payload_request = payload['data']['request']

        self.assertEqual(payload_request['method'], 'POST')
        self.assertEqual(payload_request['user_ip'], 'testclient')
        self.assertEqual(payload_request['url'], 'http://testserver/')
        self.assertDictEqual(payload_request['POST'], expected_form)
        self.assertDictEqual(
            payload_request['headers'],
            {
                'accept': '*/*',
                'accept-encoding': 'gzip, deflate',
                'connection': 'keep-alive',
                'content-length': str(len(expected_body)),
                'content-type': 'application/x-www-form-urlencoded',
                'host': 'testserver',
                'user-agent': 'testclient',
            },
        )

    @mock.patch('rollbar._check_config', return_value=True)
    @mock.patch('rollbar.send_payload')
    def test_should_add_framework_version_to_payload(self, mock_send_payload, *mocks):
        import fastapi
        from fastapi import FastAPI
        import rollbar
        from rollbar.contrib.fastapi.routing import add_to as rollbar_add_to

        self.assertIsNone(rollbar.BASE_DATA_HOOK)

        app = FastAPI()
        rollbar_add_to(app)

        rollbar.report_exc_info()

        mock_send_payload.assert_called_once()
        payload = mock_send_payload.call_args[0][0]

        self.assertIn('fastapi', payload['data']['framework'])
        self.assertIn(fastapi.__version__, payload['data']['framework'])

    @mock.patch('rollbar.lib._async.report_exc_info', new_callable=AsyncMock)
    @mock.patch('rollbar.report_exc_info')
    def test_should_use_async_report_exc_info_if_default_handler(
        self, sync_report_exc_info, async_report_exc_info
    ):
        from fastapi import FastAPI
        import rollbar
        from rollbar.contrib.fastapi.routing import add_to as rollbar_add_to

        try:
            from fastapi.testclient import TestClient
        except ImportError:  # Added in FastAPI v0.51.0+
            from starlette.testclient import TestClient

        rollbar.SETTINGS['handler'] = 'default'

        app = FastAPI()
        rollbar_add_to(app)

        @app.get('/')
        async def root():
            1 / 0

        client = TestClient(app)
        with self.assertRaises(ZeroDivisionError):
            client.get('/')

        async_report_exc_info.assert_called_once()
        sync_report_exc_info.assert_not_called()

    @mock.patch('rollbar.lib._async.report_exc_info', new_callable=AsyncMock)
    @mock.patch('rollbar.report_exc_info')
    def test_should_use_async_report_exc_info_if_any_async_handler(
        self, sync_report_exc_info, async_report_exc_info
    ):
        from fastapi import FastAPI
        import rollbar
        from rollbar.contrib.fastapi.routing import add_to as rollbar_add_to

        try:
            from fastapi.testclient import TestClient
        except ImportError:  # Added in FastAPI v0.51.0+
            from starlette.testclient import TestClient

        rollbar.SETTINGS['handler'] = 'httpx'

        app = FastAPI()
        rollbar_add_to(app)

        @app.get('/')
        async def root():
            1 / 0

        client = TestClient(app)
        with self.assertRaises(ZeroDivisionError):
            client.get('/')

        async_report_exc_info.assert_called_once()
        sync_report_exc_info.assert_not_called()

    @mock.patch('logging.Logger.warning')
    @mock.patch('rollbar.lib._async.report_exc_info', new_callable=AsyncMock)
    @mock.patch('rollbar.report_exc_info')
    def test_should_use_sync_report_exc_info_if_non_async_handlers(
        self, sync_report_exc_info, async_report_exc_info, mock_log
    ):
        from fastapi import FastAPI
        import rollbar
        from rollbar.contrib.fastapi.routing import add_to as rollbar_add_to

        try:
            from fastapi.testclient import TestClient
        except ImportError:  # Added in FastAPI v0.51.0+
            from starlette.testclient import TestClient

        rollbar.SETTINGS['handler'] = 'threading'

        app = FastAPI()
        rollbar_add_to(app)

        @app.get('/')
        async def root():
            1 / 0

        client = TestClient(app)
        with self.assertRaises(ZeroDivisionError):
            client.get('/')

        sync_report_exc_info.assert_called_once()
        async_report_exc_info.assert_not_called()
        mock_log.assert_called_once_with(
            'Failed to report asynchronously. Trying to report synchronously.'
        )

    def test_should_enable_loading_route_handler_if_fastapi_version_is_sufficient(self):
        from fastapi import FastAPI
        from fastapi.routing import APIRoute
        from rollbar.contrib.fastapi.routing import add_to as rollbar_add_to
        from rollbar.contrib.fastapi.routing import RollbarLoggingRoute

        self.assertTrue(ALLOWED_FASTAPI_VERSION)
        app = FastAPI()
        old_route_class = app.router.route_class
        self.assertEqual(old_route_class, APIRoute)

        new_route_class = rollbar_add_to(app)

        self.assertNotEqual(new_route_class, old_route_class)
        self.assertEqual(app.router.route_class, new_route_class)
        self.assertEqual(app.router.route_class, RollbarLoggingRoute)

    def test_should_enable_loading_route_handler_before_adding_routes_to_app(self):
        from fastapi import FastAPI
        from rollbar.contrib.fastapi.routing import add_to as rollbar_add_to
        from rollbar.contrib.fastapi.routing import RollbarLoggingRoute

        app = FastAPI()
        old_route_class = app.router.route_class
        self.assertEqual(len(app.routes), 4)

        new_route_class = rollbar_add_to(app)

        self.assertNotEqual(new_route_class, old_route_class)
        self.assertEqual(app.router.route_class, new_route_class)
        self.assertEqual(len(app.routes), 4)

        @app.get('/')
        async def read_root():
            ...

        self.assertEqual(app.router.route_class, new_route_class)
        self.assertEqual(app.router.route_class, RollbarLoggingRoute)
        self.assertEqual(len(app.routes), 5)

    @mock.patch('logging.Logger.error')
    def test_should_disable_loading_route_handler_after_adding_routes_to_app(
        self, mock_log
    ):
        from fastapi import FastAPI
        from rollbar.contrib.fastapi.routing import add_to as rollbar_add_to

        app = FastAPI()
        old_route_class = app.router.route_class
        self.assertEqual(len(app.routes), 4)

        @app.get('/')
        async def read_root():
            ...

        self.assertEqual(len(app.routes), 5)

        new_route_class = rollbar_add_to(app)

        self.assertEqual(len(app.routes), 5)
        self.assertIsNone(new_route_class)
        self.assertEqual(app.router.route_class, old_route_class)
        mock_log.assert_called_once_with(
            'RollbarLoggingRoute must to be added to a bare router'
            ' (before adding routes). See docs for more details.'
        )

    def test_should_enable_loading_route_handler_before_adding_routes_to_router(self):
        from fastapi import APIRouter, FastAPI
        from rollbar.contrib.fastapi.routing import add_to as rollbar_add_to
        from rollbar.contrib.fastapi.routing import RollbarLoggingRoute

        app = FastAPI()
        router = APIRouter()

        old_app_route_class = app.router.route_class
        old_router_route_class = router.route_class

        self.assertEqual(len(app.routes), 4)
        self.assertEqual(len(router.routes), 0)

        new_route_class = rollbar_add_to(router)

        self.assertNotEqual(new_route_class, old_router_route_class)
        self.assertEqual(router.route_class, new_route_class)
        self.assertEqual(router.route_class, RollbarLoggingRoute)
        self.assertEqual(app.router.route_class, old_app_route_class)
        self.assertEqual(len(app.routes), 4)
        self.assertEqual(len(router.routes), 0)

        @router.get('/')
        async def read_root():
            ...

        app.include_router(router)

        self.assertEqual(router.route_class, new_route_class)
        self.assertEqual(len(app.routes), 5)

    @mock.patch('logging.Logger.error')
    def test_should_disable_loading_route_handler_after_adding_routes_to_router(
        self, mock_log
    ):
        from fastapi import APIRouter, FastAPI
        from rollbar.contrib.fastapi.routing import add_to as rollbar_add_to

        app = FastAPI()
        router = APIRouter()

        old_app_route_class = app.router.route_class
        old_router_route_class = router.route_class

        self.assertEqual(len(app.routes), 4)
        self.assertEqual(len(router.routes), 0)

        @router.get('/')
        async def read_root():
            ...

        app.include_router(router)
        self.assertEqual(len(app.routes), 5)
        self.assertEqual(len(router.routes), 1)

        new_route_class = rollbar_add_to(app)

        self.assertEqual(len(app.routes), 5)
        self.assertEqual(len(router.routes), 1)
        self.assertIsNone(new_route_class)
        self.assertEqual(app.router.route_class, old_app_route_class)
        self.assertEqual(router.route_class, old_router_route_class)
        mock_log.assert_called_once_with(
            'RollbarLoggingRoute must to be added to a bare router'
            ' (before adding routes). See docs for more details.'
        )

    def test_should_enable_loading_route_handler_for_multiple_routers(self):
        from fastapi import APIRouter, FastAPI
        from rollbar.contrib.fastapi.routing import add_to as rollbar_add_to
        from rollbar.contrib.fastapi.routing import RollbarLoggingRoute

        app = FastAPI()
        router1 = APIRouter()
        router2 = APIRouter()
        router3 = APIRouter()

        old_app_route_class = app.router.route_class
        old_router1_route_class = router1.route_class
        old_router2_route_class = router2.route_class
        old_router3_route_class = router3.route_class

        self.assertEqual(len(app.routes), 4)
        self.assertEqual(len(router1.routes), 0)
        self.assertEqual(len(router2.routes), 0)
        self.assertEqual(len(router3.routes), 0)

        new_router1_route_class = rollbar_add_to(router1)
        new_router2_route_class = rollbar_add_to(router2)

        self.assertNotEqual(new_router1_route_class, old_router1_route_class)
        self.assertNotEqual(new_router2_route_class, old_router2_route_class)
        self.assertEqual(router1.route_class, RollbarLoggingRoute)
        self.assertEqual(router2.route_class, RollbarLoggingRoute)
        self.assertEqual(router1.route_class, new_router1_route_class)
        self.assertEqual(router2.route_class, new_router2_route_class)
        self.assertEqual(router3.route_class, old_router3_route_class)
        self.assertEqual(app.router.route_class, old_app_route_class)
        self.assertEqual(len(app.routes), 4)
        self.assertEqual(len(router1.routes), 0)
        self.assertEqual(len(router2.routes), 0)
        self.assertEqual(len(router3.routes), 0)

        @router1.get('/')
        async def read1():
            ...

        @router2.get('/')
        async def read2():
            ...

        @router3.get('/')
        async def read3():
            ...

        app.include_router(router1)
        app.include_router(router2)
        app.include_router(router3)

        self.assertEqual(router1.route_class, new_router1_route_class)
        self.assertEqual(router2.route_class, new_router2_route_class)
        self.assertEqual(router3.route_class, old_router3_route_class)
        self.assertEqual(len(app.routes), 7)

    def test_should_enable_loading_route_handler_for_fastapi_app(self):
        from fastapi import FastAPI
        from fastapi.routing import APIRoute
        from rollbar.contrib.fastapi.routing import add_to as rollbar_add_to
        from rollbar.contrib.fastapi.routing import RollbarLoggingRoute

        app = FastAPI()
        old_route_class = app.router.route_class
        self.assertEqual(old_route_class, APIRoute)

        new_route_class = rollbar_add_to(app)

        self.assertNotEqual(new_route_class, old_route_class)
        self.assertEqual(app.router.route_class, RollbarLoggingRoute)
        self.assertEqual(app.router.route_class, new_route_class)

    def test_should_enable_loading_route_handler_for_fastapi_router(self):
        from fastapi import APIRouter
        from fastapi.routing import APIRoute
        from rollbar.contrib.fastapi.routing import add_to as rollbar_add_to
        from rollbar.contrib.fastapi.routing import RollbarLoggingRoute

        router = APIRouter()
        old_route_class = router.route_class
        self.assertEqual(old_route_class, APIRoute)

        new_route_class = rollbar_add_to(router)

        self.assertNotEqual(new_route_class, old_route_class)
        self.assertEqual(router.route_class, RollbarLoggingRoute)
        self.assertEqual(router.route_class, new_route_class)

    @mock.patch('logging.Logger.error')
    def test_should_disable_loading_route_handler_for_unknown_app(self, mock_log):
        from rollbar.contrib.fastapi.routing import add_to as rollbar_add_to

        class UnkownRouter:
            route_class = None

        class UnknownApp:
            routes = []
            router = UnkownRouter()

        app = UnknownApp()
        old_route_class = app.router.route_class

        new_route_class = rollbar_add_to(app)

        self.assertIsNone(new_route_class)
        self.assertEqual(app.router.route_class, old_route_class)
        mock_log.assert_called_once_with(
            'Error adding RollbarLoggingRoute to application.'
        )

    @mock.patch('logging.Logger.error')
    def test_should_disable_loading_route_handler_for_unknown_router(self, mock_log):
        from rollbar.contrib.fastapi.routing import add_to as rollbar_add_to

        class UnknownRouter:
            routes = []
            route_class = None

        router = UnknownRouter()
        old_route_class = router.route_class

        new_route_class = rollbar_add_to(router)

        self.assertIsNone(new_route_class)
        self.assertEqual(router.route_class, old_route_class)
        mock_log.assert_called_once_with(
            'Error adding RollbarLoggingRoute to application.'
        )

    def test_should_warn_if_middleware_in_use(self):
        from fastapi import FastAPI
        from rollbar.contrib.fastapi.routing import add_to as rollbar_add_to
        from rollbar.contrib.fastapi import ReporterMiddleware as FastAPIMiddleware
        from rollbar.contrib.starlette import ReporterMiddleware as StarletteMiddleware
        from rollbar.contrib.asgi import ReporterMiddleware as ASGIMiddleware

        for middleware in (FastAPIMiddleware, StarletteMiddleware, ASGIMiddleware):
            with mock.patch('logging.Logger.warning') as mock_log:
                app = FastAPI()
                app.add_middleware(middleware)

                rollbar_add_to(app)

                mock_log.assert_called_once_with(
                    f'Detected middleware installed {[middleware]}'
                    ' while loading Rollbar route handler.'
                    ' This can cause in duplicate occurrences.'
                )

    @unittest.skipUnless(
        sys.version_info >= (3, 6), 'Global request access requires Python 3.6+'
    )
    @mock.patch('rollbar.contrib.fastapi.routing.store_current_request')
    def test_should_store_current_request(self, store_current_request):
        from fastapi import FastAPI
        from rollbar.contrib.fastapi.routing import add_to as rollbar_add_to

        try:
            from fastapi.testclient import TestClient
        except ImportError:  # Added in FastAPI v0.51.0+
            from starlette.testclient import TestClient

        expected_scope = {
            'client': ['testclient', 50000],
            'headers': [
                (b'host', b'testserver'),
                (b'user-agent', b'testclient'),
                (b'accept-encoding', b'gzip, deflate'),
                (b'accept', b'*/*'),
                (b'connection', b'keep-alive'),
            ],
            'http_version': '1.1',
            'method': 'GET',
            'path': '/',
            'query_string': b'',
            'root_path': '',
            'scheme': 'http',
            'server': ['testserver', 80],
            'type': 'http',
        }

        app = FastAPI()
        rollbar_add_to(app)

        @app.get('/')
        async def read_root():
            ...

        client = TestClient(app)
        client.get('/')

        store_current_request.assert_called_once()

        scope = store_current_request.call_args[0][0]
        self.assertDictContainsSubset(expected_scope, scope)

    @unittest.skipUnless(
        sys.version_info >= (3, 6), 'Global request access is supported in Python 3.6+'
    )
    def test_should_return_current_request(self):
        from fastapi import FastAPI
        from rollbar.contrib.fastapi import get_current_request
        from rollbar.contrib.fastapi.routing import add_to as rollbar_add_to

        try:
            from fastapi import Request
            from fastapi.testclient import TestClient
        except ImportError:  # Added in FastAPI v0.51.0+
            from starlette.requests import Request
            from starlette.testclient import TestClient

        app = FastAPI()
        rollbar_add_to(app)

        @app.get('/')
        async def read_root(original_request: Request):
            request = get_current_request()

            self.assertEqual(request, original_request)

        client = TestClient(app)
        client.get('/')

    @mock.patch('rollbar.contrib.starlette.requests.ContextVar', None)
    @mock.patch('logging.Logger.error')
    def test_should_not_return_current_request_for_older_python(self, mock_log):
        from fastapi import FastAPI
        from rollbar.contrib.fastapi import get_current_request
        from rollbar.contrib.fastapi.routing import add_to as rollbar_add_to

        try:
            from fastapi import Request
            from fastapi.testclient import TestClient
        except ImportError:  # Added in FastAPI v0.51.0+
            from starlette.requests import Request
            from starlette.testclient import TestClient

        app = FastAPI()
        rollbar_add_to(app)

        @app.get('/')
        async def read_root(original_request: Request):
            request = get_current_request()

            self.assertIsNone(request)
            self.assertNotEqual(request, original_request)
            mock_log.assert_called_once_with(
                'Python 3.7+ (or aiocontextvars package)'
                ' is required to receive current request.'
            )

        client = TestClient(app)
        client.get('/')

    def test_should_support_type_hints(self):
        from typing import Optional, Type, Union
        from fastapi import APIRouter, FastAPI
        from fastapi.routing import APIRoute
        import rollbar.contrib.fastapi.routing

        self.assertDictEqual(
            rollbar.contrib.fastapi.routing.add_to.__annotations__,
            {
                'app_or_router': Union[FastAPI, APIRouter],
                'return': Optional[Type[APIRoute]],
            },
        )



import inspect
import sys

import unittest

from rollbar.test import BaseTest

ALLOWED_PYTHON_VERSION = sys.version_info >= (3, 5)


@unittest.skipUnless(ALLOWED_PYTHON_VERSION, 'ASGI implementation requires Python3.5+')
class ASGISpecTest(BaseTest):
    def test_asgi_v3_middleware_is_single_callable_coroutine(self):
        from rollbar.contrib.asgi import ReporterMiddleware

        app = ReporterMiddleware(object)

        self.assertFalse(inspect.isclass(app))
        self.assertTrue(hasattr(app, '__call__'))
        self.assertTrue(inspect.iscoroutinefunction(app.__call__))

    def test_asgi_v3_app_signature(self):
        from rollbar.contrib.asgi import ReporterMiddleware

        app = ReporterMiddleware(object)
        app_args = inspect.getfullargspec(app).args

        self.assertListEqual(app_args, ['self', 'scope', 'receive', 'send'])



from rollbar.lib.traverse import traverse

from rollbar.test import BaseTest


class NamedTuple(tuple):
    """
    Modeled after NamedTuple and KeyedTuple from SQLAlchemy 0.7 and 0.8.
    """

    def __new__(cls, vals, labels=None):
        t = tuple.__new__(cls, vals)
        if labels:
            t.__dict__.update(zip(labels, vals))
            t._labels = labels
        return t

    def keys(self):
        return [l for l in self._labels if l is not None]


class RollbarTraverseTest(BaseTest):
    """
    Objects that appear to be a namedtuple, like SQLAlchemy's KeyedTuple,
    will cause an Exception while identifying them if they don't implement
    the _make method.
    """

    def setUp(self):
        self.tuple = NamedTuple((1, 2, 3), labels=["one", "two", "three"])

    def test_base_case(self):
        self.assertEqual(traverse(self.tuple), (1, 2, 3))

    def test_bad_object(self):
        setattr(self.tuple, "_fields", "not quite a named tuple")
        self.assertEqual(traverse(self.tuple), (1, 2, 3))



import unittest
import sys

from rollbar.test import BaseTest

ALLOWED_PYTHON_VERSION = sys.version_info >= (3, 5)


@unittest.skipUnless(ALLOWED_PYTHON_VERSION, 'ASGI implementation requires Python3.5+')
class IntegrationTest(BaseTest):
    def test_should_integrate_if__integrate_defined(self):
        from rollbar.contrib.asgi.integration import IntegrationBase

        called = False  # cannot patch local objects

        class Integration(IntegrationBase):
            def _integrate(self):
                nonlocal called
                called = True

        self.assertTrue(hasattr(Integration, '_integrate'))

        obj = Integration()

        self.assertTrue(called)
        self.assertTrue(hasattr(obj, '_integrate'))

    def test_should_not_fail_if__integrate_not_exists(self):
        from rollbar.contrib.asgi.integration import IntegrationBase

        class WrongIntegration(IntegrationBase):
            ...

        self.assertFalse(hasattr(WrongIntegration, '_integrate'))

        obj = WrongIntegration()

        self.assertFalse(hasattr(obj, '_integrate'))



import copy
import sys

from unittest import mock

import unittest

import rollbar
from rollbar.lib._async import AsyncMock
from rollbar.test import BaseTest

ALLOWED_PYTHON_VERSION = sys.version_info >= (3, 6)


@unittest.skipUnless(ALLOWED_PYTHON_VERSION, 'Async support requires Python3.6+')
class AsyncLibTest(BaseTest):
    default_settings = copy.deepcopy(rollbar.SETTINGS)

    def setUp(self):
        self.access_token = 'aaaabbbbccccddddeeeeffff00001111'
        rollbar.SETTINGS = copy.deepcopy(self.default_settings)
        rollbar._initialized = False
        rollbar.init(self.access_token, handler='async')

    @mock.patch('rollbar.send_payload')
    def test_report_exception(self, send_payload):
        from rollbar.lib._async import report_exc_info, run

        def _raise():
            try:
                raise Exception('foo')
            except:
                return run(report_exc_info())

        uuid = _raise()

        send_payload.assert_called_once()

        payload = send_payload.call_args[0][0]

        self.assertEqual(payload['data']['uuid'], uuid)
        self.assertEqual(payload['access_token'], self.access_token)
        self.assertIn('body', payload['data'])
        self.assertIn('trace', payload['data']['body'])
        self.assertNotIn('trace_chain', payload['data']['body'])
        self.assertIn('exception', payload['data']['body']['trace'])
        self.assertEqual(
            payload['data']['body']['trace']['exception']['message'], 'foo'
        )
        self.assertEqual(
            payload['data']['body']['trace']['exception']['class'], 'Exception'
        )

        self.assertNotIn('argspec', payload['data']['body']['trace']['frames'][-1])
        self.assertNotIn('varargspec', payload['data']['body']['trace']['frames'][-1])
        self.assertNotIn('keywordspec', payload['data']['body']['trace']['frames'][-1])
        self.assertIn('locals', payload['data']['body']['trace']['frames'][-1])

    @mock.patch('rollbar.send_payload')
    def test_report_messsage(self, send_payload):
        from rollbar.lib._async import report_message, run

        uuid = run(report_message('foo'))

        send_payload.assert_called_once()

        payload = send_payload.call_args[0][0]

        self.assertEqual(payload['data']['uuid'], uuid)
        self.assertEqual(payload['access_token'], self.access_token)
        self.assertIn('body', payload['data'])
        self.assertIn('message', payload['data']['body'])
        self.assertIn('body', payload['data']['body']['message'])
        self.assertEqual(payload['data']['body']['message']['body'], 'foo')

    @mock.patch('rollbar.report_exc_info')
    def test_should_run_rollbar_report_exc_info(self, rollbar_report_exc_info):
        from rollbar.lib._async import report_exc_info, run

        try:
            raise Exception()
        except Exception:
            run(
                report_exc_info(
                    'exc_info',
                    'request',
                    'extra_data',
                    'payload_data',
                    'level_data',
                    foo='bar',
                )
            )

        rollbar_report_exc_info.assert_called_once_with(
            'exc_info', 'request', 'extra_data', 'payload_data', 'level_data', foo='bar'
        )

    @mock.patch('rollbar.report_message')
    def test_should_run_rollbar_report_message(self, rollbar_report_message):
        from rollbar.lib._async import report_message, run

        run(report_message('message', 'level', 'request', 'extra_data', 'payload_data'))

        rollbar_report_message.assert_called_once_with(
            'message', 'level', 'request', 'extra_data', 'payload_data'
        )

    @mock.patch('logging.Logger.warning')
    @mock.patch('rollbar._send_payload_async')
    def test_report_exc_info_should_use_async_handler_regardless_of_settings(
        self, mock__send_payload_async, mock_log
    ):
        import rollbar
        from rollbar.lib._async import report_exc_info, run

        rollbar.SETTINGS['handler'] = 'thread'
        self.assertEqual(rollbar.SETTINGS['handler'], 'thread')

        run(report_exc_info())

        self.assertEqual(rollbar.SETTINGS['handler'], 'thread')
        mock__send_payload_async.assert_called_once()
        mock_log.assert_called_once_with(
            'Running coroutines requires async compatible handler. Switching to default async handler.'
        )

    @mock.patch('logging.Logger.warning')
    @mock.patch('rollbar._send_payload_async')
    def test_report_message_should_use_async_handler_regardless_of_settings(
        self, mock__send_payload_async, mock_log
    ):
        import rollbar
        from rollbar.lib._async import report_message, run

        rollbar.SETTINGS['handler'] = 'thread'
        self.assertEqual(rollbar.SETTINGS['handler'], 'thread')

        run(report_message('foo'))

        self.assertEqual(rollbar.SETTINGS['handler'], 'thread')
        mock__send_payload_async.assert_called_once()
        mock_log.assert_called_once_with(
            'Running coroutines requires async compatible handler. Switching to default async handler.'
        )

    @mock.patch('logging.Logger.warning')
    @mock.patch('rollbar._send_payload_async')
    def test_report_exc_info_should_allow_async_handler(
        self, mock__send_payload_async, mock_log
    ):
        import rollbar
        from rollbar.lib._async import report_exc_info, run

        rollbar.SETTINGS['handler'] = 'async'
        self.assertEqual(rollbar.SETTINGS['handler'], 'async')

        run(report_exc_info())

        self.assertEqual(rollbar.SETTINGS['handler'], 'async')
        mock__send_payload_async.assert_called_once()
        mock_log.assert_not_called()

    @mock.patch('logging.Logger.warning')
    @mock.patch('rollbar._send_payload_async')
    def test_report_message_should_allow_async_handler(
        self, mock__send_payload_async, mock_log
    ):
        import rollbar
        from rollbar.lib._async import report_message, run

        rollbar.SETTINGS['handler'] = 'async'
        self.assertEqual(rollbar.SETTINGS['handler'], 'async')

        run(report_message('foo'))

        self.assertEqual(rollbar.SETTINGS['handler'], 'async')
        mock__send_payload_async.assert_called_once()
        mock_log.assert_not_called()

    @mock.patch('logging.Logger.warning')
    @mock.patch('rollbar._send_payload_httpx')
    def test_report_exc_info_should_allow_httpx_handler(
        self, mock__send_payload_httpx, mock_log
    ):
        import rollbar
        from rollbar.lib._async import report_exc_info, run

        rollbar.SETTINGS['handler'] = 'httpx'
        self.assertEqual(rollbar.SETTINGS['handler'], 'httpx')

        run(report_exc_info())

        self.assertEqual(rollbar.SETTINGS['handler'], 'httpx')
        mock__send_payload_httpx.assert_called_once()
        mock_log.assert_not_called()

    @mock.patch('logging.Logger.warning')
    @mock.patch('rollbar._send_payload_httpx')
    def test_report_message_should_allow_httpx_handler(
        self, mock__send_payload_httpx, mock_log
    ):
        import rollbar
        from rollbar.lib._async import report_message, run

        rollbar.SETTINGS['handler'] = 'httpx'
        self.assertEqual(rollbar.SETTINGS['handler'], 'httpx')

        run(report_message('foo', 'error'))

        self.assertEqual(rollbar.SETTINGS['handler'], 'httpx')
        mock__send_payload_httpx.assert_called_once()
        mock_log.assert_not_called()

    @mock.patch('logging.Logger.warning')
    def test_ctx_manager_should_use_async_handler(self, mock_log):
        import rollbar
        from rollbar.lib._async import AsyncHandler

        rollbar.SETTINGS['handler'] = 'thread'
        with AsyncHandler() as handler:
            self.assertEqual(handler, 'async')
            mock_log.assert_called_once_with(
                'Running coroutines requires async compatible handler.'
                ' Switching to default async handler.'
            )
        rollbar.SETTINGS['handler'] = 'thread'

    @mock.patch('logging.Logger.warning')
    def test_ctx_manager_should_use_global_handler_if_contextvar_is_not_supported(
        self, mock_log
    ):
        import rollbar
        import rollbar.lib._async
        from rollbar.lib._async import AsyncHandler

        try:
            # simulate missing `contextvars` module
            _ctx_handler = rollbar.lib._async._ctx_handler
            rollbar.lib._async._ctx_handler = None

            rollbar.SETTINGS['handler'] = 'thread'
            self.assertEqual(rollbar.SETTINGS['handler'], 'thread')

            with AsyncHandler() as handler:
                self.assertEqual(handler, 'thread')
                mock_log.assert_not_called()

            self.assertEqual(rollbar.SETTINGS['handler'], 'thread')
        finally:
            # restore original _ctx_handler
            rollbar.lib._async._ctx_handler = _ctx_handler

    @mock.patch('logging.Logger.warning')
    def test_ctx_manager_should_not_substitute_global_handler(self, mock_log):
        import rollbar
        from rollbar.lib._async import AsyncHandler

        rollbar.SETTINGS['handler'] = 'thread'
        self.assertEqual(rollbar.SETTINGS['handler'], 'thread')

        with AsyncHandler() as handler:
            self.assertEqual(handler, 'async')
            self.assertEqual(rollbar.SETTINGS['handler'], 'thread')

        self.assertEqual(rollbar.SETTINGS['handler'], 'thread')
        mock_log.assert_called_once_with(
            'Running coroutines requires async compatible handler.'
            ' Switching to default async handler.'
        )

    @mock.patch('rollbar._send_payload_httpx')
    @mock.patch('rollbar._send_payload_async')
    def test_report_exc_info_message_should_allow_multiple_async_handlers(
        self, mock__send_payload_async, mock__send_payload_httpx
    ):
        import asyncio
        import rollbar
        from rollbar.lib._async import report_exc_info, run

        async def report(handler):
            rollbar.SETTINGS['handler'] = handler
            try:
                raise Exception('foo')
            except:
                await report_exc_info()

        async def send_reports():
            await asyncio.gather(report('async'), report('httpx'))

        run(send_reports())

        mock__send_payload_async.assert_called_once()
        mock__send_payload_httpx.assert_called_once()

    @mock.patch('rollbar._send_payload_httpx')
    @mock.patch('rollbar._send_payload_async')
    def test_report_message_should_allow_multiple_async_handlers(
        self, mock__send_payload_async, mock__send_payload_httpx
    ):
        import asyncio
        import rollbar
        from rollbar.lib._async import report_message, run

        async def report(handler):
            rollbar.SETTINGS['handler'] = handler
            await report_message('foo')

        async def send_reports():
            await asyncio.gather(report('async'), report('httpx'))

        run(send_reports())

        mock__send_payload_async.assert_called_once()
        mock__send_payload_httpx.assert_called_once()

    @mock.patch('rollbar._send_payload')
    @mock.patch('rollbar._send_payload_thread')
    @mock.patch('rollbar._send_payload_httpx')
    @mock.patch('rollbar._send_payload_async')
    def test_report_exc_info_should_allow_multiple_handlers(
        self,
        mock__send_payload_async,
        mock__send_payload_httpx,
        mock__send_payload_thread,
        mock__send_payload,
    ):
        import asyncio
        import rollbar
        from rollbar.lib._async import report_exc_info, run

        async def async_report(handler):
            rollbar.SETTINGS['handler'] = handler
            try:
                raise Exception('foo')
            except:
                await report_exc_info()

        async def sync_report(handler):
            rollbar.SETTINGS['handler'] = handler
            try:
                raise Exception('foo')
            except:
                rollbar.report_exc_info()

        async def send_reports():
            await asyncio.gather(
                sync_report('thread'),
                async_report('httpx'),
                sync_report('blocking'),
                async_report('async'),
            )

        run(send_reports())

        mock__send_payload_async.assert_called_once()
        mock__send_payload_httpx.assert_called_once()
        mock__send_payload_thread.assert_called_once()
        mock__send_payload.assert_called_once()

    @mock.patch('rollbar._send_payload')
    @mock.patch('rollbar._send_payload_thread')
    @mock.patch('rollbar._send_payload_httpx')
    @mock.patch('rollbar._send_payload_async')
    def test_report_message_should_allow_multiple_handlers(
        self,
        mock__send_payload_async,
        mock__send_payload_httpx,
        mock__send_payload_thread,
        mock__send_payload,
    ):
        import asyncio
        import rollbar
        from rollbar.lib._async import report_message, run

        async def async_report(handler):
            rollbar.SETTINGS['handler'] = handler
            await report_message('foo')

        async def sync_report(handler):
            rollbar.SETTINGS['handler'] = handler
            rollbar.report_message('foo')

        async def send_reports():
            await asyncio.gather(
                sync_report('thread'),
                async_report('httpx'),
                sync_report('blocking'),
                async_report('async'),
            )

        run(send_reports())

        mock__send_payload_async.assert_called_once()
        mock__send_payload_httpx.assert_called_once()
        mock__send_payload_thread.assert_called_once()
        mock__send_payload.assert_called_once()

    @mock.patch('logging.Logger.warning')
    @mock.patch('rollbar._send_payload_thread')
    @mock.patch('rollbar._send_payload_async')
    def test_report_exc_info_should_allow_multiple_handlers_with_threads(
        self,
        mock__send_payload_async,
        mock__send_payload_thread,
        mock_log,
    ):
        import time
        import threading
        import rollbar
        from rollbar.lib._async import report_exc_info, run

        async def async_report():
            try:
                raise Exception('foo')
            except:
                await report_exc_info()

        def sync_report():
            # give a chance to execute async_report() first
            time.sleep(0.1)

            try:
                raise Exception('foo')
            except:
                rollbar.report_exc_info()

        rollbar.SETTINGS['handler'] = 'thread'
        t1 = threading.Thread(target=run, args=(async_report(),))
        t2 = threading.Thread(target=sync_report)

        t1.start()
        t2.start()
        t1.join()
        t2.join()

        mock__send_payload_async.assert_called_once()
        mock__send_payload_thread.assert_called_once()
        mock_log.assert_called_once_with(
            'Running coroutines requires async compatible handler. Switching to default async handler.'
        )

    @mock.patch('logging.Logger.warning')
    @mock.patch('rollbar._send_payload_thread')
    @mock.patch('rollbar._send_payload_async')
    def test_report_message_should_allow_multiple_handlers_with_threads(
        self,
        mock__send_payload_async,
        mock__send_payload_thread,
        mock_log,
    ):
        import time
        import threading
        import rollbar
        from rollbar.lib._async import report_message, run

        async def async_report():
            await report_message('foo')

        def sync_report():
            # give a chance to execute async_report() first
            time.sleep(0.1)
            rollbar.report_message('foo')

        rollbar.SETTINGS['handler'] = 'thread'
        t1 = threading.Thread(target=run, args=(async_report(),))
        t2 = threading.Thread(target=sync_report)

        t1.start()
        t2.start()
        t1.join()
        t2.join()

        mock__send_payload_async.assert_called_once()
        mock__send_payload_thread.assert_called_once()
        mock_log.assert_called_once_with(
            'Running coroutines requires async compatible handler. Switching to default async handler.'
        )

    @mock.patch('rollbar.lib._async.report_exc_info', new_callable=AsyncMock)
    def test_should_try_report_with_async_handler(self, async_report_exc_info):
        import rollbar
        from rollbar.lib._async import run, try_report

        rollbar.SETTINGS['handler'] = 'async'
        self.assertEqual(rollbar.SETTINGS['handler'], 'async')

        run(try_report())

        async_report_exc_info.assert_called_once()

    @mock.patch('rollbar.lib._async.report_exc_info', new_callable=AsyncMock)
    def test_should_try_report_if_default_handler(self, async_report_exc_info):
        import rollbar
        from rollbar.lib._async import run, try_report

        rollbar.SETTINGS['handler'] = 'default'
        self.assertEqual(rollbar.SETTINGS['handler'], 'default')

        run(try_report())

        async_report_exc_info.assert_called_once()

    @mock.patch('rollbar.lib._async.report_exc_info', new_callable=AsyncMock)
    def test_should_not_try_report_with_async_handler_if_non_async_handler(
        self, async_report_exc_info
    ):
        import rollbar
        from rollbar.lib._async import RollbarAsyncError, run, try_report

        rollbar.SETTINGS['handler'] = 'thread'
        self.assertEqual(rollbar.SETTINGS['handler'], 'thread')

        with self.assertRaises(RollbarAsyncError):
            run(try_report())

        async_report_exc_info.assert_not_called()

    @mock.patch('rollbar.lib._async.httpx', None)
    def test_try_report_should_raise_exc_if_httpx_package_is_missing(self):
        import rollbar
        from rollbar.lib._async import RollbarAsyncError, run, try_report

        rollbar.SETTINGS['handler'] = 'httpx'
        self.assertEqual(rollbar.SETTINGS['handler'], 'httpx')

        with self.assertRaises(RollbarAsyncError):
            run(try_report())

    @mock.patch('asyncio.ensure_future')
    def test_should_schedule_task_in_event_loop(self, ensure_future):
        from rollbar.lib._async import call_later, coroutine

        try:
            if sys.version_info >= (3, 7):
                with mock.patch('asyncio.create_task') as create_task:
                    coro = coroutine()
                    call_later(coro)

                    create_task.assert_called_once_with(coro)
                    ensure_future.assert_not_called()
            else:
                coro = coroutine()
                call_later(coro)

                ensure_future.assert_called_once_with(coro)
        finally:
            # make sure the coroutine is closed to avoid RuntimeWarning by calling
            # coroutine without awaiting it later
            coro.close()



try:
    from django.core.exceptions import ImproperlyConfigured
except ImportError:
    ImproperlyConfigured = RuntimeError

try:
    from rest_framework.views import exception_handler as _exception_handler
except (ImportError, ImproperlyConfigured):
    _exception_handler = None


def post_exception_handler(exc, context):
    # This is to be used with the Django REST Framework (DRF) as its
    # global exception handler.  It replaces the POST data of the Django
    # request with the parsed data from the DRF.  This is necessary
    # because we cannot read the request data/stream more than once.
    # This will allow us to see the parsed POST params in the rollbar
    # exception log.

    if _exception_handler is None:
        raise ImproperlyConfigured(
            'Could not import rest_framework.views.exception_handler')

    try:
        context['request']._request.POST = context['request'].data
    except Exception:
        pass

    return _exception_handler(exc, context)



import requests
import threading


_local = threading.local()


def _session():
    if hasattr(_local, 'session'):
        return _local.session
    _local.session = requests.Session()
    return _local.session


def _get_proxy_cfg(kw):
    proxy = kw.pop('proxy', None)
    proxy_user = kw.pop('proxy_user', None)
    proxy_password = kw.pop('proxy_password', None)
    if proxy and proxy_user and proxy_password:
        return {
            'http': 'http://{}:{}@{}'.format(proxy_user, proxy_password, proxy),
            'https': 'http://{}:{}@{}'.format(proxy_user, proxy_password, proxy),
        }
    elif proxy:
        return {
            'http': 'http://{}'.format(proxy),
            'https': 'http://{}'.format(proxy),
        }


def configure_pool(**kw):
    keys = ['pool_connections', 'pool_maxsize', 'max_retries']
    args = {k: kw[k] for k in keys if kw.get(k, None) is not None}
    if len(args) == 0:
        return
    https_adapter = requests.adapters.HTTPAdapter(**args)
    http_adapter = requests.adapters.HTTPAdapter(**args)
    _session().mount('https://', https_adapter)
    _session().mount('http://', http_adapter)


def post(*args, **kw):
    proxies = _get_proxy_cfg(kw)
    return _session().post(*args, proxies=proxies, **kw)


def get(*args, **kw):
    proxies = _get_proxy_cfg(kw)
    return _session().get(*args, proxies=proxies, **kw)


__all__ = ['post', 'get', 'configure_pool']



from array import array
import collections
import itertools
import reprlib

from collections.abc import Mapping

from rollbar.lib import (
    integer_types, key_in, number_types, sequence_types,
    string_types)
from rollbar.lib.transform import Transform


_type_name_mapping = {
    'string': string_types,
    'long': integer_types,
    'mapping': Mapping,
    'list': list,
    'tuple': tuple,
    'set': set,
    'frozenset': frozenset,
    'array': array,
    'deque': collections.deque,
    'other': None
}


class ShortenerTransform(Transform):
    def __init__(self, safe_repr=True, keys=None, **sizes):
        super(ShortenerTransform, self).__init__()
        self.safe_repr = safe_repr
        self.keys = keys
        self._repr = reprlib.Repr()

        for name, size in sizes.items():
            setattr(self._repr, name, size)

    def _get_max_size(self, obj):
        for name, _type in _type_name_mapping.items():
            # Special case for dicts since we are using collections.abc.Mapping
            # to provide better type checking for dict-like objects
            if name == 'mapping':
                name = 'dict'

            if _type and isinstance(obj, _type):
                return getattr(self._repr, 'max%s' % name)

        return self._repr.maxother

    def _shorten_sequence(self, obj, max_keys):
        _len = len(obj)
        if _len <= max_keys:
            return obj

        return self._repr.repr(obj)

    def _shorten_mapping(self, obj, max_keys):
        _len = len(obj)
        if _len <= max_keys:
            return obj

        return {k: obj[k] for k in itertools.islice(obj.keys(), max_keys)}

    def _shorten_basic(self, obj, max_len):
        val = str(obj)
        if len(val) <= max_len:
            return obj

        return self._repr.repr(obj)

    def _shorten_other(self, obj):
        if obj is None:
            return None

        if self.safe_repr:
            obj = str(obj)

        return self._repr.repr(obj)

    def _shorten(self, val):
        max_size = self._get_max_size(val)

        if isinstance(val, dict):
            return self._shorten_mapping(val, max_size)
        if isinstance(val, (string_types, sequence_types)):
            return self._shorten_sequence(val, max_size)

        if isinstance(val, number_types):
            return self._shorten_basic(val, self._repr.maxlong)

        return self._shorten_other(val)

    def _should_shorten(self, val, key):
        if not key:
            return False

        return key_in(key, self.keys)

    def default(self, o, key=None):
        if self._should_shorten(o, key):
            return self._shorten(o)

        return super(ShortenerTransform, self).default(o, key=key)


__all__ = ['ShortenerTransform']



from rollbar.lib.transforms.scrub import ScrubTransform


class RedactRef(object):
    pass


REDACT_REF = RedactRef()


class ScrubRedactTransform(ScrubTransform):
    def default(self, o, key=None):
        if o is REDACT_REF:
            return self.redact(o)

        return super(ScrubRedactTransform, self).default(o, key=key)


__all__ = ['ScrubRedactTransform']



import math

from rollbar.lib import binary_type, string_types
from rollbar.lib import (
    circular_reference_label, float_infinity_label, float_nan_label,
    undecodable_object_label, unencodable_object_label)

from rollbar.lib.transform import Transform


class SerializableTransform(Transform):
    def __init__(self, safe_repr=True, safelist_types=None):
        super(SerializableTransform, self).__init__()
        self.safe_repr = safe_repr
        self.safelist = set(safelist_types or [])

    def transform_circular_reference(self, o, key=None, ref_key=None):
        return circular_reference_label(o, ref_key)

    def transform_namedtuple(self, o, key=None):
        tuple_dict = o._asdict()
        transformed_dict = self.transform_dict(tuple_dict, key=key)
        new_vals = []
        for field in tuple_dict:
            new_vals.append(transformed_dict[field])

        return '<%s>' % str(o._make(new_vals))

    def transform_number(self, o, key=None):
        if math.isnan(o):
            return float_nan_label(o)
        elif math.isinf(o):
            return float_infinity_label(o)
        else:
            return o

    def transform_bytes(self, o, key=None):
        try:
            o.decode('utf8')
        except UnicodeDecodeError:
            return undecodable_object_label(o)
        else:
            return repr(o)

    def transform_unicode(self, o, key=None):
        try:
            o.encode('utf8')
        except UnicodeEncodeError:
            return unencodable_object_label(o)
        else:
            return o

    def transform_dict(self, o, key=None):
        ret = {}
        for k, v in o.items():
            if isinstance(k, string_types) or isinstance(k, binary_type):
                if isinstance(k, bytes):
                    new_k = self.transform_bytes(k)
                else:
                    new_k = self.transform_unicode(k)
            else:
                new_k = str(k)

            ret[new_k] = v

        return super(SerializableTransform, self).transform_dict(ret, key=key)

    def transform_custom(self, o, key=None):
        if o is None:
            return None

        # Best to be very careful when we call user code in the middle of
        # preparing a stack trace. So we put a try/except around it all.
        try:
            if any(filter(lambda x: isinstance(o, x), self.safelist)):
                try:
                    return repr(o)
                except TypeError:
                    pass

            # If self.safe_repr is False, use repr() to serialize the object
            if not self.safe_repr:
                try:
                    return repr(o)
                except TypeError:
                    pass

            # Otherwise, just use the type name
            return str(type(o))

        except Exception as e:
            exc_str = ''
            try:
                exc_str = str(e)
            except Exception as e2:
                exc_str = '[%s while calling str(%s)]' % (
                    e2.__class__.__name__, e.__class__.__name__)
            return '<%s in %s.__repr__: %s>' % (
                e.__class__.__name__, o.__class__.__name__, exc_str)


__all__ = ['SerializableTransform']



from rollbar.lib.transform import Transform
from rollbar.lib import (
    number_types,
    type_info,
)


def do_transform(transform, type_name, val, key=None, **kw):
    fn = getattr(transform, "transform_%s" % type_name, transform.transform_custom)
    val = fn(val, key=key, **kw)

    return val


def string_handler(transform, s, key=None):
    if isinstance(s, bytes):
        return do_transform(transform, "bytes", s, key=key)
    elif isinstance(s, str):
        return do_transform(transform, "unicode", s, key=key)


def default_handler(transform, o, key=None):
    if isinstance(o, bool):
        return do_transform(transform, "boolean", o, key=key)

    # There is a quirk in the current version (1.1.6) of the enum
    # backport enum34 which causes it to not have the same
    # behavior as Python 3.4+. One way to identify IntEnums is that
    # they are instances of numbers but not number types.
    if isinstance(o, number_types):
        if type(o) not in number_types:
            return do_transform(transform, "custom", o, key=key)
        else:
            return do_transform(transform, "number", o, key=key)

    return do_transform(transform, "custom", o, key=key)


handlers = {
    type_info.STRING: string_handler,
    type_info.TUPLE: lambda transform, o, key=None: do_transform(
        transform, "tuple", o, key=key
    ),
    type_info.NAMEDTUPLE: lambda transform, o, key=None: do_transform(
        transform, "namedtuple", o, key=key
    ),
    type_info.LIST: lambda transform, o, key=None: do_transform(
        transform, "list", o, key=key
    ),
    type_info.SET: lambda transform, o, key=None: do_transform(
        transform, "set", o, key=key
    ),
    type_info.MAPPING: lambda transform, o, key=None: do_transform(
        transform, "dict", o, key=key
    ),
    type_info.CIRCULAR: lambda transform, o, key=None, ref_key=None: do_transform(
        transform, "circular_reference", o, key=key, ref_key=ref_key
    ),
    type_info.DEFAULT: default_handler,
}


class BatchedTransform(Transform):
    def __init__(self, transforms):
        super(BatchedTransform, self).__init__()
        self._transforms = transforms

    def default(self, o, key=None):
        for transform in self._transforms:
            node_type = type_info.get_type(o)
            handler = handlers.get(node_type, handlers.get(type_info.DEFAULT))
            o = handler(transform, o, key=key)

        return o


__all__ = ["BatchedTransform"]



from rollbar.lib.transforms import transform
from rollbar.lib.transform import Transform
from rollbar.lib.traverse import traverse

from rollbar.test import BaseTest


class TrackingTransformer(Transform):
    def __init__(self):
        self.got = []

    def default(self, o, key=None):
        self.got.append((o, key))
        return o


class BatchedTransformTest(BaseTest):
    def assertTrackingTransform(self, input):
        tracking_transformer = TrackingTransformer()

        transforms = [
            tracking_transformer,
            tracking_transformer,
        ]

        transform(input, transforms, batch_transforms=True)

        want = []

        def dup_watch_handler(o, key=None):
            want.append((o, key))
            want.append((o, key))
            return o

        traverse(
            input,
            string_handler=dup_watch_handler,
            tuple_handler=dup_watch_handler,
            namedtuple_handler=dup_watch_handler,
            list_handler=dup_watch_handler,
            set_handler=dup_watch_handler,
            mapping_handler=dup_watch_handler,
            default_handler=dup_watch_handler,
            circular_reference_handler=dup_watch_handler,
        )

        self.assertEqual(want, tracking_transformer.got)

    def test_number(self):
        self.assertTrackingTransform(1)

    def test_flat_list(self):
        self.assertTrackingTransform([0, 1, 2, 3])

    def test_flat_tuple(self):
        self.assertTrackingTransform((0, 1, 2, 3))

    def test_nested_object(self):
        self.assertTrackingTransform((0, [1, 2], {"a": 3, "b": (4, 5)}))



import re
import os.path
from setuptools import setup, find_packages

HERE = os.path.abspath(os.path.dirname(__file__))

README_PATH = os.path.join(HERE, 'README.md')
try:
    with open(README_PATH) as fd:
        README = fd.read()
except IOError:
    README = ''

INIT_PATH = os.path.join(HERE, 'rollbar/__init__.py')
with open(INIT_PATH) as fd:
    INIT_DATA = fd.read()
    VERSION = re.search(r"^__version__ = ['\"]([^'\"]+)['\"]", INIT_DATA, re.MULTILINE).group(1)

tests_require = [
    'webob',
    'blinker',
    'httpx',
    'aiocontextvars; python_version == "3.6"'
]

setup(
    name='rollbar',
    packages=find_packages(),
    version=VERSION,
    entry_points={
        'paste.filter_app_factory': [
            'pyramid=rollbar.contrib.pyramid:create_rollbar_middleware'
        ],
        'console_scripts': ['rollbar=rollbar.cli:main']
    },
    description='Easy and powerful exception tracking with Rollbar. Send '
                'messages and exceptions with arbitrary context, get back '
                'aggregates, and debug production issues quickly.',
    long_description=README,
    long_description_content_type="text/markdown",
    author='Rollbar, Inc.',
    author_email='support@rollbar.com',
    test_suite='rollbar.test.discover',
    url='http://github.com/rollbar/pyrollbar',
    classifiers=[
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "Programming Language :: Python :: 3 :: Only",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Development Status :: 5 - Production/Stable",
        "Environment :: Web Environment",
        "Framework :: AsyncIO",
        "Framework :: Bottle",
        "Framework :: Django",
        "Framework :: Flask",
        "Framework :: Pylons",
        "Framework :: Pyramid",
        "Framework :: Twisted",
        "Intended Audience :: Developers",
        "Topic :: Internet :: WWW/HTTP",
        "Topic :: Software Development",
        "Topic :: Software Development :: Bug Tracking",
        "Topic :: Software Development :: Testing",
        "Topic :: Software Development :: Quality Assurance",
        "Topic :: System :: Logging",
        "Topic :: System :: Monitoring",
        ],
    install_requires=[
        'requests>=0.12.1',
    ],
    tests_require=tests_require,
)



import importlib
import sys

from unittest import mock

try:
    import fastapi

    FASTAPI_INSTALLED = True
except ImportError:
    FASTAPI_INSTALLED = False

import unittest

import rollbar
from rollbar.test import BaseTest

ALLOWED_PYTHON_VERSION = sys.version_info >= (3, 6)


@unittest.skipUnless(
    FASTAPI_INSTALLED and ALLOWED_PYTHON_VERSION,
    'FastAPI LoggerMiddleware requires Python3.6+',
)
class LoggerMiddlewareTest(BaseTest):
    def setUp(self):
        importlib.reload(rollbar)

    @mock.patch('rollbar._check_config', return_value=True)
    @mock.patch('rollbar.send_payload')
    def test_should_add_framework_version_to_payload(self, mock_send_payload, *mocks):
        import fastapi
        from fastapi import FastAPI
        import rollbar
        from rollbar.contrib.fastapi.logger import LoggerMiddleware

        self.assertIsNone(rollbar.BASE_DATA_HOOK)

        app = FastAPI()
        app.add_middleware(LoggerMiddleware)

        rollbar.report_exc_info()

        mock_send_payload.assert_called_once()
        payload = mock_send_payload.call_args[0][0]

        self.assertIn('fastapi', payload['data']['framework'])
        self.assertIn(fastapi.__version__, payload['data']['framework'])

    def test_should_support_type_hints(self):
        from starlette.types import Receive, Scope, Send
        import rollbar.contrib.fastapi.logger

        self.assertDictEqual(
            rollbar.contrib.fastapi.logger.LoggerMiddleware.__call__.__annotations__,
            {'scope': Scope, 'receive': Receive, 'send': Send, 'return': None},
        )

    @mock.patch('rollbar.contrib.starlette.logger.store_current_request')
    def test_should_store_current_request(self, store_current_request):
        from fastapi import FastAPI
        from rollbar.contrib.fastapi.logger import LoggerMiddleware

        try:
            from fastapi.testclient import TestClient
        except ImportError:  # Added in FastAPI v0.51.0+
            from starlette.testclient import TestClient

        expected_scope = {
            'client': ['testclient', 50000],
            'headers': [
                (b'host', b'testserver'),
                (b'user-agent', b'testclient'),
                (b'accept-encoding', b'gzip, deflate'),
                (b'accept', b'*/*'),
                (b'connection', b'keep-alive'),
            ],
            'http_version': '1.1',
            'method': 'GET',
            'path': '/',
            'query_string': b'',
            'root_path': '',
            'scheme': 'http',
            'server': ['testserver', 80],
            'type': 'http',
        }

        app = FastAPI()
        app.add_middleware(LoggerMiddleware)

        @app.get('/')
        async def read_root():
            return 'ok'

        client = TestClient(app)
        client.get('/')

        store_current_request.assert_called_once()

        scope = store_current_request.call_args[0][0]
        self.assertDictContainsSubset(expected_scope, scope)

    def test_should_return_current_request(self):
        from fastapi import FastAPI
        from starlette.requests import Request
        from rollbar.contrib.fastapi.logger import LoggerMiddleware
        from rollbar.contrib.fastapi import get_current_request

        try:
            from fastapi.testclient import TestClient
        except ImportError:  # Added in FastAPI v0.51.0+
            from starlette.testclient import TestClient

        app = FastAPI()
        app.add_middleware(LoggerMiddleware)

        @app.get('/')
        async def read_root():
            request = get_current_request()

            self.assertIsNotNone(request)
            self.assertIsInstance(request, Request)

        client = TestClient(app)
        client.get('/')

    @mock.patch('rollbar.contrib.starlette.requests.ContextVar', None)
    @mock.patch('logging.Logger.error')
    def test_should_not_return_current_request_for_older_python(self, mock_log):
        from fastapi import FastAPI
        from rollbar.contrib.fastapi.logger import LoggerMiddleware
        from rollbar.contrib.fastapi import get_current_request

        try:
            from fastapi.testclient import TestClient
        except ImportError:  # Added in FastAPI v0.51.0+
            from starlette.testclient import TestClient

        app = FastAPI()
        app.add_middleware(LoggerMiddleware)

        @app.get('/')
        async def read_root():
            self.assertIsNone(get_current_request())
            mock_log.assert_called_once_with(
                'Python 3.7+ (or aiocontextvars package)'
                ' is required to receive current request.'
            )

        client = TestClient(app)
        client.get('/')



from rollbar.lib import dict_merge, prefix_match

from rollbar.test import BaseTest


class RollbarLibTest(BaseTest):
    def test_dict_merge_not_dict(self):
        a = {'a': {'b': 42}}
        b = 99
        result = dict_merge(a, b)

        self.assertEqual(99, result)

    def test_prefix_match(self):
        key = ['password', 'argspec', '0']
        self.assertTrue(prefix_match(key, [['password']]))

    def test_prefix_match(self):
        key = ['environ', 'argspec', '0']
        self.assertFalse(prefix_match(key, [['password']]))

    def test_dict_merge_dicts_independent(self):
        a = {'a': {'b': 42}}
        b = {'x': {'y': 99}}
        result = dict_merge(a, b)

        self.assertIn('a', result)
        self.assertIn('b', result['a'])
        self.assertEqual(42, result['a']['b'])
        self.assertIn('x', result)
        self.assertIn('y', result['x'])
        self.assertEqual(99, result['x']['y'])

    def test_dict_merge_dicts(self):
        a = {'a': {'b': 42}}
        b = {'a': {'c': 99}}
        result = dict_merge(a, b)

        self.assertIn('a', result)
        self.assertIn('b', result['a'])
        self.assertIn('c', result['a'])
        self.assertEqual(42, result['a']['b'])
        self.assertEqual(99, result['a']['c'])

    def test_dict_merge_dicts_second_wins(self):
        a = {'a': {'b': 42}}
        b = {'a': {'b': 99}}
        result = dict_merge(a, b)

        self.assertIn('a', result)
        self.assertIn('b', result['a'])
        self.assertEqual(99, result['a']['b'])

    def test_dict_merge_dicts_select_poll(self):
        import select
        poll = getattr(select, 'poll', None)
        if poll is None:
            return
        p = poll()
        a = {'a': {'b': 42}}
        b = {'a': {'y': p}}
        result = dict_merge(a, b, silence_errors=True)

        self.assertIn('a', result)
        self.assertIn('b', result['a'])
        self.assertEqual(42, result['a']['b'])
        self.assertIn('y', result['a'])
        self.assertRegex(result['a']['y'], r'Uncopyable obj')



__all__ = ['add_to']

import logging
import sys
from typing import Callable, Optional, Type, Union

from fastapi import APIRouter, FastAPI, __version__
from fastapi.routing import APIRoute

try:
    from fastapi import Request, Response
except ImportError:
    # Added in FastAPI v0.51.0
    from starlette.requests import Request
    from starlette.responses import Response

import rollbar
from .utils import fastapi_min_version, get_installed_middlewares, has_bare_routing
from rollbar.contrib.asgi.integration import integrate
from rollbar.contrib.starlette.requests import store_current_request
from rollbar.lib._async import RollbarAsyncError, try_report

log = logging.getLogger(__name__)


@fastapi_min_version('0.41.0')
@integrate(framework_name=f'fastapi {__version__}')
def add_to(app_or_router: Union[FastAPI, APIRouter]) -> Optional[Type[APIRoute]]:
    """
    Adds RollbarLoggingRoute handler to the router app.

    This is the recommended way for integration with FastAPI.
    Alternatively to using middleware, the handler may fill
    more data in the payload (e.g. request body).

    app_or_router: FastAPI app or router

    Note: The route handler must be added before adding user routes

    Requirements: FastAPI v0.41.0+

    Example usage:

    from fastapi import FastAPI
    from rollbar.contrib.fastapi import add_to as rollbar_add_to

    app = FastAPI()
    rollbar_add_to(app)

    """

    if not has_bare_routing(app_or_router):
        log.error(
            'RollbarLoggingRoute must to be added to a bare router'
            ' (before adding routes). See docs for more details.'
        )
        return None

    installed_middlewares = get_installed_middlewares(app_or_router)
    if installed_middlewares:
        log.warning(
            f'Detected middleware installed {installed_middlewares}'
            ' while loading Rollbar route handler.'
            ' This can cause in duplicate occurrences.'
        )

    if isinstance(app_or_router, FastAPI):
        _add_to_app(app_or_router)
    elif isinstance(app_or_router, APIRouter):
        _add_to_router(app_or_router)
    else:
        log.error('Error adding RollbarLoggingRoute to application.')
        return None

    return RollbarLoggingRoute


class RollbarLoggingRoute(APIRoute):
    def get_route_handler(self) -> Callable:
        router_handler = super().get_route_handler()

        async def rollbar_route_handler(request: Request) -> Response:
            try:
                store_current_request(request)
                return await router_handler(request)
            except Exception:
                # FastAPI requires the `python-multipart` package to parse the content
                if not request._stream_consumed:
                    await request.body()
                await request.form()

                exc_info = sys.exc_info()

                try:
                    await try_report(exc_info, request)
                except RollbarAsyncError:
                    log.warning(
                        'Failed to report asynchronously. Trying to report synchronously.'
                    )
                    rollbar.report_exc_info(exc_info, request)
                raise

        return rollbar_route_handler


def _add_to_app(app):
    app.router.route_class = RollbarLoggingRoute


def _add_to_router(router):
    router.route_class = RollbarLoggingRoute
